@@ -6,7 +6,7 @@ use warnings;
 
 use Scalar::Util qw(blessed);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Tree::Simple::Manager::Index;
 use Tree::Simple::Manager::Exceptions;
@@ -15,6 +15,8 @@ use Tree::Parser;
 
 use Tree::Simple;
 use Tree::Simple::View::DHTML;
+
+use Storable ();
     
 sub new {
     my ($_class, @tree_configs) = @_;
@@ -33,7 +35,7 @@ sub _init {
     while (@{$tree_configs}) {
         my ($tree_name, $config) = splice @{$tree_configs}, 0, 2;
         
-        (exists ${$config}{tree_root} && ${$config}{tree_file_path})
+        (exists ${$config}{tree_root})
             || throw Tree::Simple::Manager::InsufficientArguments "missing the required keys for '$tree_name' config";
         
         (!exists ${$self->{trees}}{$tree_name})
@@ -43,26 +45,10 @@ sub _init {
     
         my $root_tree = $config->{tree_root};
         (blessed($root_tree) && $root_tree->isa('Tree::Simple'))
-            || throw Tree::Simple::Manager::IncorrectObjectType "The 'root_tree' must be a Tree::Simple instance (or a subclass of it)";
-        
-        my $tree;
-        eval {
-            my $tp = Tree::Parser->new($root_tree);
-            $tp->setInput($config->{tree_file_path});
-            if (exists ${$config}{tree_parse_filter}) {
-                (ref($config->{tree_parse_filter}) eq 'CODE')
-                    || throw Tree::Simple::Manager::IncorrectObjectType "a 'tree_parse_filter' must be a code ref";
-                $tp->setParseFilter($self->_parseFilterWrapper(ref($root_tree) => $config->{tree_parse_filter}));
-            }
-            else {
-                $tp->setParseFilter($self->_getDefaultParseFilter(ref($root_tree)));
-            }
-            $tp->parse();
-            $tree = $tp->getTree();
-        };
-        if ($@) {
-            throw Tree::Simple::Manager::OperationFailed "unable to parse tree file '" . $config->{tree_file_path}. "'" => $@;
-        }
+            || throw Tree::Simple::Manager::IncorrectObjectType "The 'root_tree' must be a Tree::Simple instance (or a subclass of it)";    
+
+        # load from the file or something
+        my $tree = $self->_loadTree($tree_name => $config);    
         
         # by default we use our Index module
         
@@ -92,6 +78,68 @@ sub _init {
     }
 }
 
+sub _loadTree {
+    my ($self, $tree_name, $config) = @_;
+    
+    if (exists $config->{tree_cache_path} && -e $config->{tree_cache_path}) {
+        my $tree;
+        eval {
+            $tree = Storable::retrieve($config->{tree_cache_path});
+        };
+        if ($@) {
+            warn "Unable to load tree cache, removing cache tree";
+            unlink $config->{tree_cache_path};
+            warn "Attempting to load tree with parser";
+        }    
+        else {
+            $self->{trees_loaded_from_cache}->{$tree_name}++;
+            return $tree;        
+        }
+    }
+
+    (exists $config->{tree_file_path})
+        || throw Tree::Simple::Manager::InsufficientArguments "missing the required keys for '$tree_name' config";
+    
+    my $tree;
+    eval {
+        my $tp = Tree::Parser->new($config->{tree_root});
+        $tp->setInput($config->{tree_file_path});
+        if (exists $config->{tree_parse_filter}) {
+            (ref($config->{tree_parse_filter}) eq 'CODE')
+                || throw Tree::Simple::Manager::IncorrectObjectType "a 'tree_parse_filter' must be a code ref";
+            $tp->setParseFilter(
+                $self->_parseFilterWrapper(
+                    ref($config->{tree_root}), 
+                    $config->{tree_parse_filter}
+                )
+            );
+        }
+        else {
+            $tp->setParseFilter(
+                $self->_getDefaultParseFilter(
+                    ref($config->{tree_root})
+                )
+            );
+        }
+        $tp->parse();
+        $tree = $tp->getTree();
+    };
+    if ($@) {
+        throw Tree::Simple::Manager::OperationFailed "unable to parse tree file '" . $config->{tree_file_path}. "'" => $@;
+    }    
+    
+    if (exists $config->{tree_cache_path}) {
+        eval {
+            Storable::store($tree, $config->{tree_cache_path});
+        };
+        if ($@) {
+            warn "Unable to store tree cache ... sorry";
+        }    
+    }    
+    
+    return $tree;
+}
+
 sub _parseFilterWrapper {
     my ($self, $tree_type, $filter) = @_;
     return sub {
@@ -163,6 +211,12 @@ sub getNewTreeView {
     return $tree_view_class->new($self->getRootTree($tree_name), @view_args);
 }
 
+sub isTreeLoadedFromCache {
+    my ($self, $tree_name) = @_;
+    exists $self->{trees_loaded_from_cache}->{$tree_name} && 
+           $self->{trees_loaded_from_cache}->{$tree_name};
+}
+
 1;
 
 __END__
@@ -195,9 +249,21 @@ Tree::Simple::Manager - A class for managing multiple Tree::Simple hierarchies
 
 =head1 DESCRIPTION
 
-This is a class for managing multiple Tree::Simple hierarchies at a time. It integrates several Tree::Simple classes together to attempt to make things easier to manage. This is the first release of this module. It is currently tailored to my current needs, and will likely get more flexible later on. If you want to use it, and need it to work differently, let me know and I can try to help, or you can submit a patch.
+This is a class for managing multiple Tree::Simple hierarchies at a 
+time. It integrates several Tree::Simple classes together to attempt 
+to make things easier to manage. This is the third release of this 
+module. It is currently tailored to my current needs, and will likely 
+get more flexible later on. If you want to use it, and need it to 
+work differently, let me know and I can try to help, or you can 
+submit a patch.
 
-The basic idea of this module is that you can load and store Tree::Simple hierarchies by name. You use L<Tree::Parser> to load the hierarchy from disk, the tree is then indexed for fast node retrieval by L<Tree::Simple::Manager::Index>. If you need a L<Tree::Simple::View> of the tree, you can create one with this class, or get the L<Tree::Simple::View> subclass which is associated with this tree.
+The basic idea of this module is that you can load and store 
+Tree::Simple hierarchies by name. You use L<Tree::Parser> to load 
+the hierarchy from disk, the tree is then indexed for fast node 
+retrieval by L<Tree::Simple::Manager::Index>. If you need a 
+L<Tree::Simple::View> of the tree, you can create one with this 
+class, or get the L<Tree::Simple::View> subclass which is 
+associated with this tree.
 
 =head1 METHODS
 
@@ -205,7 +271,9 @@ The basic idea of this module is that you can load and store Tree::Simple hierar
 
 =item B<new (%tree_configs)>
 
-This will load all the tree heirachies from disk, index them. The config format is show above in L<SYNOPSIS>, and described in detail below: 
+This will load all the tree heirachies from disk, index them. 
+The config format is show above in L<SYNOPSIS>, and described 
+in detail below: 
 
 B<Required Fields>
 
@@ -213,11 +281,13 @@ B<Required Fields>
 
 =item I<tree_root> 
 
-This must be a Tree::Simple object (or a subclass of Tree::Simple) it will serve as the root of this particular tree. 
+This must be a Tree::Simple object (or a subclass of Tree::Simple) 
+it will serve as the root of this particular tree. 
 
 =item I<tree_file_path> 
 
-This must be a valid path to a tree file which L<Tree::Parser> will understand.
+This must be a valid path to a tree file which L<Tree::Parser> 
+will understand.
 
 =back
 
@@ -227,15 +297,22 @@ B<Optional Fields>
 
 =item I<tree_index>
 
-This must be a package name for a L<Tree::Simple::Manager::Index> subclass. The default is L<Tree::Simple::Manager::Index>.
+This must be a package name for a L<Tree::Simple::Manager::Index> 
+subclass. The default is L<Tree::Simple::Manager::Index>.
 
 =item I<tree_view>
 
-This must be a package name for a L<Tree::Simple::View> subclass. The default is L<Tree::Simple::View::DHTML>.
+This must be a package name for a L<Tree::Simple::View> subclass. 
+The default is L<Tree::Simple::View::DHTML>.
 
-=item <tree_parse_filter>
+=item I<tree_parse_filter>
 
-This must be a subroutine reference which is compatible with L<Tree::Parser>'s parse filters. It's first argument is an L<Array::Iterator> object which has all the lines in the tree file. It's second argument is the tree class name as specified in the C<tree_root> field. Here is an example custom parse filter:
+This must be a subroutine reference which is compatible 
+with L<Tree::Parser>'s parse filters. It's first argument 
+is an L<Array::Iterator> object which has all the lines in the 
+tree file. It's second argument is the tree class name as 
+specified in the C<tree_root> field. Here is an example 
+custom parse filter:
 
   # this will parse tree files formated like this:
   # uid:node
@@ -260,13 +337,22 @@ The default parse filter will parse tree files which look like this:
   uid            node
   uid        node
 
-Where the UID is first, followed by a tab, then either the node value or more tabs to indicate the tree depth.
+Where the UID is first, followed by a tab, then either the node value 
+or more tabs to indicate the tree depth.
+
+=item I<tree_cache_path>
+
+This must be a valid file path, and can be used to cache a parsed 
+tree. It serializes the tree using Storable and then if a valid cache 
+file is present, it will us that instead of re-parsing. This can 
+potentially save a B<lot> of time during startup for large trees.
 
 =back
 
 =item B<getTreeList>
 
-This will return a list of names of the tree hierarchies currently being managed.
+This will return a list of names of the tree hierarchies currently 
+being managed.
 
 =item B<getRootTree ($tree_name)>
 
@@ -274,19 +360,29 @@ This will return the root of the tree found at C<$tree_name>.
 
 =item B<getTreeIndex ($tree_name)>
 
-This will return the Tree::Simple::Manager::Index object found at C<$tree_name>.
+This will return the Tree::Simple::Manager::Index object found 
+at C<$tree_name>.
 
 =item B<getTreeByID ($tree_name, $tree_id)>
 
-This will ask the tree index (found at C<$tree_name>) for the tree whose id is C<$tree_id>.
+This will ask the tree index (found at C<$tree_name>) for the 
+tree whose id is C<$tree_id>.
 
 =item B<getTreeViewClass ($tree_name)>
 
-This will return the Tree::Simple::View class associated with C<$tree_name>.
+This will return the Tree::Simple::View class associated with 
+C<$tree_name>.
 
 =item B<getNewTreeView ($tree_name, @view_args)>
 
-This will return an instance of the Tree::Simple::View class associated with C<$tree_name>, passing in the C<@view_args> to the view constructor.
+This will return an instance of the Tree::Simple::View class 
+associated with C<$tree_name>, passing in the C<@view_args> 
+to the view constructor.
+
+=item B<isTreeLoadedFromCache ($tree_name)>
+
+This will return true if the tree has been loaded from cache with 
+the tree_cache_path option.
 
 =back
 
@@ -296,33 +392,25 @@ This will return an instance of the Tree::Simple::View class associated with C<$
 
 =item I<A bunch of stuff>
 
-This is the second release of this module. I wrote it a while ago for a specific project, and decided it was useful outside of that project as well. I am sure this can be expanded much further, and I am sure I will discover ways to do that as I use it more and more. 
+This is the second release of this module. I wrote it a while 
+ago for a specific project, and decided it was useful outside of 
+that project as well. I am sure this can be expanded much further, 
+and I am sure I will discover ways to do that as I use it more 
+and more. 
 
 Suggestions, comments and even patches are always welcome.
 
 =item I<Support the pre-built filters of Tree::Parser>
 
-Tree::Parser has a number of pre-built filters, I would like be able to take advantage of them with this module.
+Tree::Parser has a number of pre-built filters, I would like be able 
+to take advantage of them with this module.
 
 =back
 
 =head1 BUGS
 
-None that I am aware of. Of course, if you find a bug, let me know, and I will be sure to fix it. 
-
-=head1 CODE COVERAGE
-
-I use B<Devel::Cover> to test the code coverage of my tests, below is the B<Devel::Cover> report on this module test suite.
-
- --------------------------------- ------ ------ ------ ------ ------ ------ ------
- File                                stmt branch   cond    sub    pod   time  total
- --------------------------------- ------ ------ ------ ------ ------ ------ ------
- Tree/Simple/Manager.pm             100.0  100.0   66.7  100.0  100.0   78.2   97.4
- Tree/Simple/Manager/Exceptions.pm  100.0    n/a    n/a  100.0    n/a    4.7  100.0
- Tree/Simple/Manager/Index.pm       100.0   75.0   66.7  100.0  100.0   17.1   94.1
- --------------------------------- ------ ------ ------ ------ ------ ------ ------
- Total                              100.0   95.8   66.7  100.0  100.0  100.0   96.7
- --------------------------------- ------ ------ ------ ------ ------ ------ ------
+None that I am aware of. Of course, if you find a bug, let me know, 
+and I will be sure to fix it. 
 
 =head1 SEE ALSO
 
@@ -342,7 +430,7 @@ stevan little, E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2004 by Infinity Interactive, Inc.
+Copyright 2004-2007 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
